///////////////////////////////////////////////////////////////////////////////
//
// 3D Utility Library
// Copyright (C) 2006 Tobias Minch
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Written by
//   Tobias Minich - belgabor@gmx.de
//
///////////////////////////////////////////////////////////////////////////////

#ifndef SCNFILE_H_INCLUDED
#define SCNFILE_H_INCLUDED

#include <string>
#include "RCT3Structs.h"
#include "3DLoader.h"

#define VERSION_CSCNFILE 3

#define CSCNFILE_NO_ERROR               0x00000000
#define CSCNFILE_ERROR_FILENAME         0x00000001
#define CSCNFILE_ERROR_OBJNOTFOUND      0x00000002
#define CSCNFILE_ERROR_MESH_FIX         0x00000004
#define CSCNFILE_ERROR_MESH_MISMATCH    0x00000008
#define CSCNFILE_ERROR_MESH_CONFUSION   0x00000010
#define CSCNFILE_ERROR_MESH_POOF        0x00000020

class cSCNFile {
public:
    std::string filename;
    long version;
    long error;
    std::string objfile;
    std::vector<cMeshStruct> meshstructs;
    std::vector<cFlexiTexture> flexitextures;
    std::vector<std::string> references;
    std::vector<cEffectPoint> effectpoints;
	std::vector<D3DMATRIX> transforms;
	std::vector<std::string> transformnames;
	cSIVSettings sivsettings;
    std::vector<std::string> tmpmeshlist;


	cSCNFile(): filename(""),version(VERSION_CSCNFILE),error(CSCNFILE_NO_ERROR),objfile("") {};
	cSCNFile(std::string l_filename): filename(l_filename),error(CSCNFILE_NO_ERROR),objfile("") {Load();};
	bool Load();
	bool LoadObject(bool reset = true);
	bool Save();
	bool Ok() {return version <= VERSION_CSCNFILE;};
	bool Error() {return error;};
private:
    cMeshStruct MakeMeshStruct(c3DLoader *obj, unsigned int j);
    void ResetAndReload(c3DLoader *obj);
    void Fixup(c3DLoader *obj);
};

#endif // SCNFILE_H_INCLUDED
